#!/bin/sh
# ============================================================================
#
# This file is part of the 'MetaRepository for zypkgs' package
# 
# This program is free software; you can redistribute it and/or modify it 
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# Author: Mijzelf <Mijzelf@live.com>
#
# ============================================================================
#

RELDIR=` dirname $0 `
CONFIG=/usr/local/zy-pkgs/etc/MetaRepository.conf
. ${CONFIG}

FindApp()
{
	case $1 in
		wget)
			[ ! -z "$WGET" ] && echo $WGET && return
			;;
		md5sum)
			[ ! -z "$MD5SUM" ] && echo $MD5SUM && return
			;;
	esac
	
	echo -n "FindApp $@ " >&2
	unset app
	
	for path in /bin/ /sbin/ /usr/bin/ /usr/sbin/ /usr/local/bin/ /usr/local/sbin/ /ffp/bin/ /ffp/sbin/
	do
		[ -x ${path}$1 ] && echo found ${path}$1 >&2 && app=${path}$1 && break
	done

	[ -z $app ] && echo Failed! >&2 && exit 1
	
	case $1 in
		wget)
			WGET=$app
			;;
		md5sum)
			MD5SUM=$app
			;;
	esac
	echo $app
}

Fetch()
{
	from=$1
	to=$2
	[ -z "$WGET" ] && WGET=` FindApp wget `
	
	echo -n "Fetch $@ " >&2
	
	case $1 in
		http://*)
			$WGET -t 3 -q $from -O $to
			local res=$?
			[ $res -eq 0 ] && echo succeeded >&2 && return $res
			echo failed >&2 
			return $res
			;;
		https://*)
			$WGET -t 3 -q $from -O $to --no-check-certificate # Yeah, I know this sucks
			local res=$?
			[ $res -eq 0 ] && echo succeeded >&2 && return $res
			echo failed >&2 
			return $res
			;;
		ftp://*)
			local pasv="--passive-ftp"
			[ "$NOPASV" = "true" ] && pasv="" || echo -n "using PASV " >&2
			
			$WGET -t 3 -q $pasv $from -O $to
			local res=$?
			[ $res -eq 0 ] && echo succeeded >&2 && return $res
			echo failed >&2 
			return $res
			;;
			
	esac
	
	if [ "$to" = "-" ] ; then
		[ -f $from ] && cat $from && echo succeeded >&2 && return 0
		echo failed >&2
		return 1
	fi
	
	[ -f $from ] && cp $from $to && echo succeeded >&2 && return 0
	echo failed >&2
	return 1
}

URLtoFilename()
{
	echo $1 | sed 's|[/|:]|_|g'
}

StripLeadingAndTrailingSpaces()
{
	[ -z "$1" ] && return
	echo "$@"
}

RemoveComments()
{
    echo "$@" | sed 's/#.*$//'
}

ReadConfig()
{
	echo ReadConfig $@ >&2
	
	local field=$1
	local comments=0
	[ "${field}" = "Comments" ] && comments=1 && field=$2 
	local myfield=0
	
	while read line
	do
		line=` StripLeadingAndTrailingSpaces "$line" `
		[ $comments -eq 0 ] && line=` RemoveComments "$line" `
		
		[ "$line" = "" ] && continue
		
		case "${line}" in
			\[${field}]*)
				echo -e "\tStart of block" >&2
				myfield=1
				continue
				;;
			\[*)
				[ $myfield -eq 1 ] && echo -e "\tEnd of block" >&2 && return
				;;
		esac
		
		[ $myfield -eq 1 ] && echo "$line" && echo -e "\t$line" >&2
	done <${CONFIG}
}

HtmlHeader()
{
	echo Content-type: text/html
	echo 
}

TarHeader()
{
	echo TarHeader >&2
        cat >&1 <<__EOF__
HTTP/1.0 200 OK
Cache-Control: private
Server: bash/2.0
Connection: Close
Content-type: application/gzip

__EOF__
}

NotFound()
{
	echo NotFound $1 >&2
	cat >&1 <<__EOF__
HTTP/1.0 404 Not found
Content-type: text/html

<html>
<head><title>404 Not Found</title></head>
<body>$1 Not Found</body>
</html>
__EOF__
}


ZYPKG_INFO_tgz()
{
	exec 2>${SCRATCH}/ZYPKG_INFO.tgz.log

	. ${RELDIR}/ZYPKG_INFO.sh

	HandleZYPKG_INFO "$1"

	local target="${SCRATCH}$1/ZYPKG_INFO.tgz"
	[ ! -f "${target}" ] && NotFound && exit 0

	TarHeader
	Fetch "${target}" -
}

PKG_INFO_tgz()
{
	exec 2>${SCRATCH}/PKG_INFO.tgz.log

	. ${RELDIR}/ZYPKG_INFO.sh

	HandlePKG_INFO "$1"

	local target="${SCRATCH}$1/PKG_INFO.tgz"
	[ ! -f "${target}" ] && NotFound && exit 0

	TarHeader
	Fetch "${target}" -
}


Index()
{
	. ${RELDIR}/index.sh
	
	Index_html
}

Main()
{
	exec 2>/dev/null
	echo "$0 $@" >&2
	printenv >&2
	echo "==========" >&2 

	[ -z REQUEST_URI ] && NotFound && exit 0

	Index
}

InvokeEngine()
{
	local path=` dirname $1 `
	local file=` basename $1 `

	[ "$file" = "ZYPKG_INFO.tgz" ] && ZYPKG_INFO_tgz "$path" && exit 0 
	[ "$file" = "PKG_INFO.tgz" ] && PKG_INFO_tgz "$path" && exit 0 
	
	local target="${SCRATCH}${1}"
	exec 2>${SCRATCH}/zypkg.log
	echo Request: $target >&2
	
	[ ! -f "${target}" ] && NotFound ${target} && exit 0

	TarHeader
	Fetch ` cat ${target} ` -
	exit 0
}

ParseInput()
{
	[ -f /tmp/MetaRepository.log ] && exec 2>>/tmp/MetaRepository.log
        local request=""

        while read line
        do
                [ " " ">" "$line" ] && [ "$request" != "" ] && InvokeEngine $request

                if echo $line | grep GET | grep HTTP >/dev/null
                then
                        request=` echo $line | awk '{print $2}' `
                        # let's do *some* security
                        request=` echo $request | sed 's|/../|/|g' `
                fi
        done

        cat >&1 <<__EOF__
HTTP/1.0 400 Bad request

__EOF__
	exit 0
}

[ "$1" = "inetd-server" ] && ParseInput

[ -f /tmp/MetaRepository.log ] && echo "` date `: $0 $@ ${REQUEST_URI}" >>/tmp/MetaRepository.log

Main

